use std::collections::HashSet;

use axum::{extract::FromRequestParts, http::request::Parts, RequestPartsExt};
use axum_extra::{
    headers::{authorization::Bearer, Authorization},
    typed_header::TypedHeaderRejectionReason,
    TypedHeader,
};
use bearer::verify_bearer;
pub use error::AuthError;

mod bearer;
mod error;

#[derive(Debug)]
pub struct Permissions<'a>(pub HashSet<&'a str>);

// Middleware for getting permissions
impl<S> FromRequestParts<S> for Permissions<'_>
where
    S: Send + Sync,
{
    type Rejection = crate::Error;

    async fn from_request_parts(parts: &mut Parts, _state: &S) -> Result<Self, Self::Rejection> {
        // First check if the request has a beaerer token to authenticate
        match parts.extract::<TypedHeader<Authorization<Bearer>>>().await {
            Ok(bearer) => {
                verify_bearer(bearer.token().to_string()).map_err(|_| AuthError::InvalidToken)?;

                let permissions = Permissions {
                    0: HashSet::from(["root"]),
                };

                return Ok(permissions);
            }
            Err(err) => match err.reason() {
                TypedHeaderRejectionReason::Missing => (),
                TypedHeaderRejectionReason::Error(_err) => {
                    return Err(AuthError::InvalidToken.into())
                }
                _ => return Err(AuthError::Unexpected.into()),
            },
        };

        Err(AuthError::Unexpected.into())
    }
}
